* Encoding: windows-1252.
*
* SPSS Command Syntax File
* Re- construction of sample design variables for EU-SILC UDB 2012, version4
*
* EU-SILC D-File (household register): 
* UDB_c12D_ver 2012-4 from 01-01-16.csv
*
* For transforming the EU-SILC CSV-data (as released by Eurostat) into SPSS-data (*.sav),
* please use the corresponding SPSS Command Syntax File published at 
* http://www.gesis.org/missy/eu/setups/EU-SILC
*
* (c) GESIS 07/05/2018
* GESIS - Leibniz Institute for the Social Sciences
* German Microdata Lab 
* Anika Herter; Heike Wirth
* http://www.gesis.org/en/institute/
*
* Contact: heike.wirth@gesis.org
* 
**************************************************************************************************************
* AIM AND PROCEDURE 
*
*    The aim of the SPSS-syntax is the reconstruction of original sample design variables psu1 
*    (primary sampling unit) and strata1 (primary strata) in the D-file of the EU-SILC cross-sectional 
*    data. They are necessary to compute standard errors but incomplete in the UDB.
*    The reconstruction procedure has been implemented and published for stata by Goedeme for 
*     waves 2005-2011 and by Goedeme and Trinidade for waves 2012-2014. Goedeme and Trinidade 
*    also documented their procedure in two articles (Goedeme 2013; Trinidade & Goedeme 2016). 
*    The structure of the SPSS-Syntax at hand follows mostly the structure of the stata codes from 
*    Trinidade & Goedeme. 
*    The first step is some data preparation where variable DB020 is renamed as country and
*     DB030 as hid. Due to anonymization, the stratification variable is not at all available in the  
*     UDB. The psu variable DB060 is missing for several countries. Therefore, these countries must 
*     be handled individually. 
*    For creating the stratification variable, it is necessary to compute a region variable for countries 
*    with regional stratification (same countries as above) based on DB040. For all other countries, 
*    region is set to zero. Strata0 is then computed by combining a numeric country code and the 
*    previously constructed region variable. Countries with self-representing psus also have to be 
*    treated individually. A psutest-variable is created based on the psu-variable of the UDB (DB060). 
*    For constructing the psu variable, this variable is combined with the stratification variable. For 
*    countries with missing values for this variable, the household id is alternatively used. Some 
*    country-specific modifications are necessary here. The variable checker is constructed to identify 
*    cases where households moved between selection to be a respondent and the interview itself. 
*    Regrouping of psus within regions is done for these cases.
*    Finally, all not required auxiliary variables are dropped and sample design variables are reviewed.
*    SPSS Complex Samples is used to define the sampling plan (csplan) and to choose desired statistical analysis 
*    (e.g. frequencies).
*
*
* For details on EU Silc standard error calculation and stata codes please visit the homepage of Tim Goedeme  
* https://timgoedeme.com/eu-silc-standard-errors/
* 
* Goedeme, T. (2013). How much Confidence can we have in EU-SILC? Complex Sample Designs and the
*     Standard Error of the Europe 2020 Poverty Indicators in Social Indicators Research, 110(1): 89-110,                                                                                          
*     doi:10.1007/s11205-011-9918-2.
* Zardo Trindade, L. and Goedeme, T. (2016) Notes on updating the EU-SILC UDB sample design variables 2012-2014,
*       CSB Working Paper 16/02, Antwerp: Herman Deleeck Centre for Social Policy, University of Antwerp.
*
* Please note: Since the SPSS syntax is based on the STATA syntax of Tim Goedeme 
* (Download: https://timgoedeme.com/eu-silc-standard-errors/), we have also adopted the corresponding comments from him
* (marked as "Goedeme 2011")
*
**************************************************************************************************************
* Open EU Silc D-file.

set unicode no.

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - .
* The following command should contain the complete path and
  name of the SPSS file, usual file extension "sav";
  Change CSAPLAN_FILENAME to your filename .
GET
  FILE='CSAPLAN_FILENAME'.

set decimal dot.


* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - .
* Please note:
* Until line 480 (Complex Sample Definition), there should be probably nothing to change. 
*  
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - .


**************************************************************************************************************
* 1. Preparation
**************************************************************************************************************
* Generate identifier.
 COMPUTE id=$CASENUM.
  FORMAT id (F8.0). 
  EXECUTE.


*generate country and hid variable.
rename variables (DB020 = country). 
execute.
autorecode country /into countrynr.
execute.
rename variables (DB030 = hid).                                                             
                                                   
* Count number of countries.
compute numcountry = 1.
if (country eq lag(country)) numcountry = 0.
execute.
frequencies numcountry.

* show country labels.
temporary.
select if numcountry = 1.
list country .

***************************************************************************************************************
* 2. Special cases that have to be handled before rest
**************************************************************************************************************

compute  psutest = DB060.                     
                                             
**************************************************************************************************************
* 2.1 BG and PL:  "DB060 not reliable in case of Bulgaria and Poland" (Goedeme 2011: 112).
**************************************************************************************************************.
do if (country='BG' or country='PL')  .
  recode psutest (0 thru 9999999 = sysmis).
end if.

**************************************************************************************************************
* 2.2 FR
**************************************************************************************************************.
* "France: change in sample design in 2010 complicates matters; no satisfactory method to identify 
*   self-representing PSUs in old sample design" (Goedeme 2011: 114).


**************************************************************************************************************
* 2.3 IT
**************************************************************************************************************.
sort cases by  country DB060 DB075.
recode DB060 (SYSMIS=99999999).
compute tester=0.
execute.
do if (DB060 eq lag(DB060) and DB075 ne lag(DB075)). 
         recode tester (0=1).
end if.
do if (id=1). 
        recode tester (0=1).
end if.
missing values tester (0).
execute.
FREQUENCIES tester.        
missing values DB060 (99999999).
missing values DB060 (99999999).
DESCRIPTIVES DB060 DB075.

sort cases by country DB060 DB075.
 AGGREGATE OUTFILE=* MODE=ADDVARIABLES
       / PRESORTED
       / BREAK  country DB060 
       / npanels=sum(tester).
recode npanels (SYSMIS=0).
frequencies npanels.
sort cases by country DB060.
compute auswahl1=0.
do if (country = "IT") and (DB060 ~= lag(DB060)).
    recode auswahl1 (0=1).
end if.
filter by auswahl1.
    frequencies npanels.
filter off.

temporary.
select if ((country = "IT")).
    frequencies npanels.

do if (npanels >=2 and country = "IT").
    recode psutest (0 thru 9999999= sysmis).
end if.
temporary.
select if ((country = "IT")).
    frequencies npanels.
do if (npanels >=2 and country = "IT").
    compute tester1 = DB060.
end if.
execute.
freq tester1.
recode tester1 (sysmis = 999999).
sort cases by tester1 .
compute groupsit = 1.
do if (tester1 = lag(tester1)).
       compute groupsit= lag(groupsit).
  else if (tester1 ne lag(tester1)).
       compute groupsit = lag(groupsit) + 1. 
end if.
execute.
DESCRIPTIVES groupsit.
**************************************************************************************************************
* 2.4 UK
***************************************************************************************************************.
do if country='UK'.
   compute cons=1.
end if.
execute.

missing values cons (-9).



sort cases by country DB060.
temporary.
select if (country = 'UK').
AGGREGATE OUTFILE=* MODE=ADDVARIABLES
    / PRESORTED
   / BREAK  country DB060 
   / nrpsu=sum(cons).
execute.
frequencies nrpsu.

temporary. 
select if (DB040 = 'UKN') .
DESCRIPTIVES nrpsu.   

sort cases by country DB040.
AGGREGATE OUTFILE=* MODE=ADDVARIABLES
   / PRESORTED
   / BREAK  country  DB040
   / maximum=max(nrpsu).
execute.

do if ((country = 'UK') and (DB040 = 'UKN') and (nrpsu=maximum)).
    recode psutest (else= sysmis).
end if.
execute.

**************************************************************************************************************
* 3. Prepare Stratification variable
**************************************************************************************************************
* "AT: DB060 completely missing - therefore also stratification ignored...
*  Also for BG and PL DB060 is not sufficiently reliable -> switch to assuming a simple random 
*  sample of households (probably under-estimation of the variance)" (Goedeme 2011: 182-183).

frequencies country.
string region0 (A6).
execute.
compute region0 = DB040.
*recode region0 (' ' ='0').
*execute.
Missing values region0 (' ').
execute.

* "Melilla (ES64) and Ceuta (ES63) must be grouped together as they are part of the same 
*   stratum."  (Goedeme 2011: 192).
recode region0 ('ES63' 'ES64' = 'ES80').  

temporary.
select if ( country= 'BE' OR country='CZ' OR country='EL'  OR country='ES'  OR country='FR'  OR country='IT'  OR country='RO').
autorecode region0 /into region0nr.
execute.
compute region1=region0nr.
recode region1 (sysmis=0).
execute.
temporary.
select if region1>0.
    descriptives region1.    

sort cases by region1.
                            

compute kons=1.
sort cases region1.
AGGREGATE OUTFILE=* MODE=ADDVARIABLES
  / presorted 
  / BREAK  kons
  / regiomin = MAX(region1).
execute.
freq regiomin.
                                                                                                                                                    
do if (npanels>=2 AND country='IT').
    COMPUTE region1a = groupsit + regiomin.
    RECODE region1a (ELSE=Copy) INTO region1.   
end if .  
execute.  
descriptives region1.

compute strata0 = countrynr*1000 + region1.
descriptives strata0.

temporary. 
select if (country='UK').
descriptives  strata0. 

sort cases country strata0.
temporary.
select if (country = "UK").
                                                                                                                                                                                 
AGGREGATE OUTFILE=* MODE=ADDVARIABLES
   / PRESORTED
   / BREAK kons
   / strat1=max(strata0).
execute.
compute stratum=strat1 + 2.
freq stratum.                                                                                                                                                                             
                                                                                                   
do if country='UK' AND MISSING(psutest).
    RECODE stratum (ELSE=Copy) INTO strata0.   
end if .                                                                                                                   
EXECUTE.
DESCRIPTIVES strata0.


**************************************************************************************************************
* 4. Prepare PSU variable
**************************************************************************************************************

COMPUTE psu0=strata0 * 100000  + hid / 100000.
EXECUTE.                                                                                                                     
DESCRIPTIVES psu0.

recode psutest (sysmis=-9999).
execute.
do if (psutest~=-9999).                                                                                                                 
     compute psu0 = strata0 * 100000 + psutest.                                                                   
end if.
MISSING VALUES psutest (-9999).
DESCRIPTIVES psu0.


**************************************************************************************************************
* 5. RE-grouping of PSUs
**************************************************************************************************************

compute checker=1.
recode checker (1=sysmis).
execute.
sort cases by country psutest hid.

vector nocheck(1) .
execute.
rename variables nocheck1=nocheck.
recode nocheck  (sysmis = -9999).
do if (misval (psutest))OR (npanels>=2 AND country='IT').
    recode nocheck (-9999=1).
end if.
execute.
missing values nocheck (-9999).
frequencies nocheck.
missing values nocheck ().

missing values psutest ().
if ((psu0 ne LAG(psu0)) and (psutest ne lag(psutest)) OR nocheck=1)  checker=0.
execute.
if ((LAG(psu0) EQ psu0) and (LAG(psutest) eq psutest) and nocheck ne 1) checker=0.
execute.
if ((LAG(psu0) ne psu0 ) and (LAG(psutest) eq psutest ) and nocheck ne 1 ) checker=1.
execute.
if ((LAG(psu0) eq psu0) and (LAG(psutest) ne psutest ) and nocheck ne 1) checker=2.
execute.
freq checker.

missing values psutest (-9999).
missing values nocheck (-9999).
frequencies nocheck.

crosstabs country by  checker.                                                                                                                                          

* reset checker to 0 if PSUs must be split across strata.
do if (country ~= 'BE' and country~='CZ' and country~='ES' and country~='FR' and country~= 'IT' and country~= 'RO').
    RECODE checker (SYSMIS=0) (1=0) (2=0).
end if.
EXECUTE.
crosstabs country by checker.

sort cases by country psu0 id. 

compute auswahl=0.
do if (country='BE' or country='CZ' or country='ES' or country='FR' or country= 'IT' or country= 'RO') and (psu0~=LAG(psu0)) .
    recode auswahl (0=1).
end if.
filter by auswahl.
crosstabs country by checker.
filter off.                                                                                                                              

crosstabs country by checker.         
                                                                                                                                            
SORT CASES  BY country.                                                                              
Temporary.
SPLIT FILE SEPARATE BY country.
SELECT IF ( checker = 1).
FREQUENCIES VARIABLES=psutest.
SPLIT FILE off.
Execute.


compute key=0.
sort cases country psutest strata0 id  .
AGGREGATE OUTFILE = * MODE=ADDVARIABLES OVERWRITE=YES
  / PRESORTED
  / BREAK = country psutest strata0
  / strata0freq = N(strata0).

AGGREGATE OUTFILE = * MODE=ADDVARIABLES OVERWRITE=YES 
 / PRESORTED
 / BREAK = country psutest 
 / aux1 = MAX( strata0freq )
 / aux2 = first(strata0)
 / aux3 = sd(strata0freq).
execute. 

IF (strata0freq = aux1) key = 1.
execute.
compute strata0_mod = key*strata0.

AGGREGATE OUTFILE = * MODE=ADDVARIABLES OVERWRITE=YES
 / PRESORTED
 / BREAK = country psutest 
 / checkmax = max(checker).

AGGREGATE OUTFILE = * MODE=ADDVARIABLES OVERWRITE=YES
 / PRESORTED
 / BREAK = country psutest 
 / strata_01 = max(strata0_mod).


do if (checkmax=0).
  compute strata1=strata0.
end if.

do if (checkmax = 1).
  compute strata1 = strata_01.
end if.

do if (aux3=0 & checkmax=1).
  compute strata1 = aux2.
end if.
execute. 

descriptives strata1 psutest psu0.

missing values psutest ().
DESCRIPTIVES psutest.

compute psu1=psu0.
execute.
do if (psutest ne -9999).
  compute psu1 =strata1*100000+psutest.
end if.
execute.
missing values psutest (-9999).

descriptives strata0 strata1 psutest psu0 psu1.


DELETE VARIABLES countrynr psutest npanels tester tester1 groupsit cons nrpsu region0 region1 strata0 
          psu0 checker nocheck id numcountry  region0nr kons regiomin region1a strat1 stratum
         auswahl auswahl1 key strata0freq aux1 aux2  aux3 strata0_mod checkmax strata_01 maximum.
         

 **************************************************************************************************************
* 6. Finalisation
**************************************************************************************************************


                                                                                                                                                                                                                                           
***   DEFINITION OF THE SAMPLE-PLAN   
* Assumptions (Approximations):
*     One-stage stratified random sample
*     Stratification: Stratification variable (strata1)
*      -  1st stage: Primary sampling units (psu1):
*     Unrestricted random sample on each selection stage
                        
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - .
* The following command should contain the complete path and
*  name your csaplan-file
* change 'CSAPLAN_FILENAME (extension .csaplan) to your filename .                    
                                                                                                                                                                                                                                                                                                                                     .
CSPLAN ANALYSIS
  /PLAN FILE='CSAPLAN_FILENAME'
  /PLANVARS ANALYSISWEIGHT=DB090       
  /SRSESTIMATOR TYPE=WOR
  /PRINT PLAN
  /DESIGN STRATA=strata1 CLUSTER=psu1 
  /ESTIMATOR TYPE=WR.
 
*EXAMPLE
* Complex Samples Crosstabs.
CSTABULATE
  /PLAN FILE='CSAPLAN_FILENAME'
  /TABLES VARIABLES=country BY DB100
  /CELLS ROWPCT
  /STATISTICS SE CIN(95)
  /MISSING SCOPE=TABLE CLASSMISSING=EXCLUDE.


rename variables (country = DB020). 
rename variables (hid = DB030).



